/*-
 * Copyright 2021 Hochschule für Technik Stuttgart
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.hft.stuttgart.citygml.viewer;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import de.hft.stuttgart.citygml.viewer.math.Vector3d;
import de.hft.stuttgart.citygml.viewer.parser.EnvelopeFoundException;

public class CityGmlHandler extends DefaultHandler {

	private boolean foundEnvelope = false;

	private Vector3d lowerCorner;
	private Vector3d upperCorner;
	private String epsg;

	private boolean nextContentIsLowerCorner = false;
	private boolean nextContentIsUpperCorner = false;

	@Override
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
		if (foundEnvelope) {
			if (qName.endsWith("lowerCorner")) {
				nextContentIsLowerCorner = true;
			} else if (qName.endsWith("upperCorner")) {
				nextContentIsUpperCorner = true;
			}
		} else if (qName.endsWith("Envelope")) {
			foundEnvelope = true;
			epsg = attributes.getValue("srsName");
		}
	}

	public Vector3d getLowerCorner() {
		return lowerCorner;
	}

	public Vector3d getUpperCorner() {
		return upperCorner;
	}

	public String getEpsg() {
		return epsg;
	}

	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		if (nextContentIsLowerCorner) {
			lowerCorner = parsePoint(ch, start, length);
			nextContentIsLowerCorner = false;
		} else if (nextContentIsUpperCorner) {
			upperCorner = parsePoint(ch, start, length);
			nextContentIsUpperCorner = false;
			throw new EnvelopeFoundException("Found envelope, stop parsing");
		}
	}

	private Vector3d parsePoint(char[] ch, int start, int length) {
		String s = new String(ch, start, length);
		String[] coords = s.split("\\s+");
		double x = Double.parseDouble(coords[0]);
		double y = Double.parseDouble(coords[1]);
		double z = Double.parseDouble(coords[2]);
		return new Vector3d(x, y, z);
	}
}
