/*-
 * Copyright 2021 Hochschule für Technik Stuttgart
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.hft.stuttgart.citygml.viewer.datastructure;

import java.util.ArrayList;
import java.util.List;

import de.hft.stuttgart.citygml.viewer.math.Vector3d;

public class Ring {

	private List<Vector3d> vertices = new ArrayList<>();

	public List<Vector3d> getVertices() {
		return vertices;
	}

	public void addPoint(Vector3d v) {
		vertices.add(v);
	}

	public Vector3d calculateNormal() {
		double[] coords = new double[3];
		for (int i = 0; i < vertices.size() - 1; i++) {
			Vector3d current = vertices.get(i + 0);
			Vector3d next = vertices.get(i + 1);
			coords[0] += (current.getZ() + next.getZ()) * (current.getY() - next.getY());
			coords[1] += (current.getX() + next.getX()) * (current.getZ() - next.getZ());
			coords[2] += (current.getY() + next.getY()) * (current.getX() - next.getX());
		}

		if (coords[0] == 0 && coords[1] == 0 && coords[2] == 0) {
			// no valid normal vector found
			if (vertices.size() < 3) {
				// no three points, return x-axis
				return new Vector3d(1, 0, 0);
			}

			Vector3d v1 = vertices.get(0);
			Vector3d v2 = vertices.get(1);
			Vector3d v3 = vertices.get(2);
			return calculateNormalWithCross(v1, v2, v3);
		}
		Vector3d v = new Vector3d(coords);
		v.normalize();
		return v;
	}

	private Vector3d calculateNormalWithCross(Vector3d v1, Vector3d v2, Vector3d v3) {
		Vector3d dir1 = v2.minus(v1);
		Vector3d dir2 = v3.minus(v1);
		Vector3d cross = dir1.cross(dir2);
		return cross.normalize();
	}
}
