/*-
 * Copyright 2021 Hochschule für Technik Stuttgart
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.hft.stuttgart.citygml.viewer.datastructure;

import de.hft.stuttgart.citygml.viewer.math.Vector3d;

/**
 * A three dimensional triangle described by three points
 * 
 * @author Matthias Betz
 *
 */
public class Triangle3d {

	private Vector3d p1;
	private Vector3d p2;
	private Vector3d p3;

	public Triangle3d(Vector3d p1, Vector3d p2, Vector3d p3) {
		this.p1 = p1;
		this.p2 = p2;
		this.p3 = p3;
	}

	public Vector3d getP1() {
		return p1;
	}

	public Vector3d getP2() {
		return p2;
	}

	public Vector3d getP3() {
		return p3;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((p1 == null) ? 0 : p1.hashCode());
		result = prime * result + ((p2 == null) ? 0 : p2.hashCode());
		result = prime * result + ((p3 == null) ? 0 : p3.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Triangle3d other = (Triangle3d) obj;
		if (p1 == null) {
			if (other.p1 != null) {
				return false;
			}
		} else if (!p1.equals(other.p1)) {
			return false;
		}
		if (p2 == null) {
			if (other.p2 != null) {
				return false;
			}
		} else if (!p2.equals(other.p2)) {
			return false;
		}
		if (p3 == null) {
			if (other.p3 != null) {
				return false;
			}
		} else if (!p3.equals(other.p3)) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "Triangle3d [p1=" + p1 + ", p2=" + p2 + ", p3=" + p3 + "]";
	}
}
