/*-
 * Copyright 2021 Hochschule für Technik Stuttgart
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.hft.stuttgart.citygml.viewer.parser;

import java.util.EventListener;

/**
 * Implementations of this listener will be informed about the (file) loading progress of ObservedInputStream
 * instances. You have to register your implementation of this interface at the ObservedInputStream instance.
 * 
 * @author Marcel Bruse
 */
public interface InputStreamListener extends EventListener {
	
	/**
	 * This method will be called by the observed input stream when a new chunk of bytes has been read. 
	 * 
	 * @param progress Should be a value between 0 and 1. 0 means no progress and 1 means loading finished. 
	 */
	public void updateProgress(float progress);
	
}
